import * as vscode from 'vscode';
import axios from 'axios';
import { exec } from 'child_process';
import * as nls from 'vscode-nls';

// Initialisiere die Lokalisierung
const localize = nls.config({ messageFormat: nls.MessageFormat.file })();

// Funktion zum Erstellen eines Pull Requests
async function createGiteaPullRequest() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage(localize('giteaClone.configMissing', 'Gitea URL or Token is not configured.'));
        return;
    }

    // Aktuellen Git-Ordner abrufen
    const currentWorkspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri?.fsPath;
    if (!currentWorkspaceFolder) {
        vscode.window.showErrorMessage(localize('giteaClone.noValidRepo', 'No valid Git repository found in the current workspace.'));
        return;
    }

    // Git-Repository-Informationen abrufen
    exec(`git config --get remote.origin.url`, { cwd: currentWorkspaceFolder }, async (error, stdout, stderr) => {
        if (error || !stdout) {
            vscode.window.showErrorMessage(localize('giteaClone.noRemoteUrl', 'Could not retrieve Git remote URL.'));
            return;
        }

        let repoUrl = stdout.trim();

        // URL umwandeln, wenn sie im SSH-Format vorliegt (git@gitea.com:owner/repo.git)
        if (repoUrl.startsWith('git@')) {
            const parts = repoUrl.split(':');
            const domain = parts[0].replace('git@', '');
            const path = parts[1].replace('.git', '');
            repoUrl = `https://${domain}/${path}`;
        } else {
            repoUrl = repoUrl.replace('.git', '');
        }

        // Extract repository owner and name from the URL
        const [owner, repo] = repoUrl.split('/').slice(-2);

        try {
            // Den letzten Commit und den Branch abrufen
            const { title, body } = await getLastCommit(currentWorkspaceFolder);
            const branch = await getCurrentBranch(currentWorkspaceFolder);

            if (!branch) {
                vscode.window.showErrorMessage(localize('giteaClone.noBranch', 'Could not determine the branch.'));
                return;
            }

            // Base-Branch über die Gitea API ermitteln
            const baseBranch = await getDefaultBranch(instanceUrl, owner, repo, token);

            // API-Request-Daten vorbereiten
            const prData = {
                title: title, // Der letzte Commit als Titel
                body: body || '', // Commit-Kommentare als Body
                head: branch, // Der aktuelle Branch als "head"
                base: baseBranch // Der ermittelte base-Branch
            };

            const config = {
                method: 'post',
                maxBodyLength: Infinity,
                url: `${instanceUrl}/api/v1/repos/${owner}/${repo}/pulls`,
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': `token ${token}`
                },
                data: JSON.stringify(prData)
            };

            // API-Request zum Erstellen des Pull Requests
            const response = await axios.request(config);

            const prUrl = response.data.html_url;

            // Öffne die URL des erstellten PRs im Browser
            vscode.env.openExternal(vscode.Uri.parse(prUrl));
            vscode.window.showInformationMessage(localize('giteaClone.pullRequestCreated', 'Pull request created successfully and opened in the browser.'));
        } catch (err: any) {
            vscode.window.showErrorMessage(localize('giteaClone.pullRequestError', `Error creating pull request: ${err.message}`));
            console.error('Error creating PR:', err);
        }
    });
}

// Hilfsfunktion, um den letzten Commit zu ermitteln
async function getLastCommit(folderPath: string): Promise<{ title: string, body: string }> {
    return new Promise((resolve, reject) => {
        exec(`git log -1 --pretty=format:"%s%n%b"`, { cwd: folderPath }, (error, stdout) => {
            if (error) {
                reject(localize('giteaClone.commitError', 'Error retrieving the last commit.'));
            } else {
                const output = stdout.split('\n');
                const title = output[0]; // Commit-Message als Titel
                const body = output.slice(1).join('\n'); // Commit-Kommentar als Body
                resolve({ title: title.trim(), body: body.trim() });
            }
        });
    });
}

// Hilfsfunktion, um den aktuellen Branch zu ermitteln
async function getCurrentBranch(folderPath: string): Promise<string> {
    return new Promise((resolve, reject) => {
        exec(`git rev-parse --abbrev-ref HEAD`, { cwd: folderPath }, (error, stdout) => {
            if (error) {
                reject(localize('giteaClone.branchError', 'Error retrieving the branch.'));
            } else {
                resolve(stdout.trim());
            }
        });
    });
}

// Hilfsfunktion, um den base-Branch über die Gitea API zu ermitteln
async function getDefaultBranch(instanceUrl: string, owner: string, repo: string, token: string): Promise<string> {
    try {
        const response = await axios.get(`${instanceUrl}/api/v1/repos/${owner}/${repo}`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200 && response.data.default_branch) {
            return response.data.default_branch;
        } else {
            vscode.window.showErrorMessage(localize('giteaClone.defaultBranchError', 'Could not determine the base branch. Defaulting to "main".'));
            return 'main';
        }
    } catch (error) {
        vscode.window.showErrorMessage(localize('giteaClone.defaultBranchError', 'Error retrieving the base branch. Defaulting to "main".'));
        return 'main';
    }
}

// Funktion zum Abrufen der Repositories des Benutzers von Gitea
async function getGiteaRepositories(): Promise<any[]> {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage(localize('giteaClone.configMissing', 'Gitea URL or Token is not configured.'));
        return [];
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user/repos`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            return response.data;
        } else {
            vscode.window.showErrorMessage(localize('giteaClone.repoError', 'Error retrieving repositories.'));
            return [];
        }
    } catch (error) {
        vscode.window.showErrorMessage(localize('giteaClone.apiError', 'Error connecting to Gitea API.'));
        console.error(error);
        return [];
    }
}

// Funktion zum Klonen eines Repositories via SSH
async function cloneGiteaRepository() {
    const repos = await getGiteaRepositories();

    if (repos.length === 0) {
        vscode.window.showInformationMessage(localize('giteaClone.noRepos', 'No repositories found.'));
        return;
    }

    const repoNames = repos.map(repo => repo.full_name);
    const selectedRepo = await vscode.window.showQuickPick(repoNames, {
        placeHolder: localize('giteaClone.selectRepo', 'Select a repository to clone')
    });

    if (!selectedRepo) {
        vscode.window.showInformationMessage(localize('giteaClone.noRepoSelected', 'No repository selected.'));
        return;
    }

    const repo = repos.find(r => r.full_name === selectedRepo);

    if (repo && repo.ssh_url) {
        const folderUri = await vscode.window.showOpenDialog({
            canSelectFiles: false,
            canSelectFolders: true,
            canSelectMany: false,
            openLabel: localize('giteaClone.selectFolder', 'Select folder to clone into')
        });

        if (folderUri && folderUri[0]) {
            const folderPath = folderUri[0].fsPath;

            vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: localize('giteaClone.cloningRepo', `Cloning ${selectedRepo}`),
                cancellable: false
            }, async (progress, token) => {
                progress.report({ message: localize('giteaClone.cloningProgress', 'Cloning in progress...'), increment: 0 });

                return new Promise<void>((resolve, reject) => {
                    exec(`git clone ${repo.ssh_url} ${folderPath}`, (error, stdout, stderr) => {
                        if (error) {
                            vscode.window.showErrorMessage(localize('giteaClone.cloneError', 'Error cloning the repository.'));
                            console.error(stderr);
                            reject(error);
                        } else {
                            progress.report({ message: localize('giteaClone.cloneComplete', 'Repository cloned successfully.'), increment: 100 });
                            vscode.window.showInformationMessage(localize('giteaClone.cloneSuccess', `Repository ${selectedRepo} cloned successfully.`));

                            // Öffne das geklonte Repository im VSCode
                            try {
                                vscode.commands.executeCommand('vscode.openFolder', vscode.Uri.file(folderPath), true)
                                    .then(() => resolve());
                            } catch (err: unknown) {
                                vscode.window.showErrorMessage(localize('giteaClone.openRepoError', 'Error opening the cloned repository.'));
                                console.error(err);
                                reject(err);
                            }
                        }
                    });
                });
            });
        } else {
            vscode.window.showInformationMessage(localize('giteaClone.noFolderSelected', 'No target folder selected.'));
        }
    } else {
        vscode.window.showErrorMessage(localize('giteaClone.noSshUrl', 'Could not find SSH clone URL.'));
    }
}

// Funktion zum Hinzufügen des Statusbar-Icons
function addStatusBarIcon() {
    const statusBar = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
    statusBar.text = `$(git-pull-request) ${localize('giteaClone.createPullRequest', 'Create Gitea PR')}`;
    statusBar.tooltip = localize('giteaClone.createPullRequestTooltip', 'Create a Pull Request in Gitea');
    statusBar.command = 'gitea.createPullRequest';
    statusBar.show(); // Sofortiges Anzeigen des Icons in der Statusleiste
    return statusBar;
}

// Funktion zur Authentifizierung bei Gitea mit dem Personal Access Token (PAT)
async function authenticateGitea() {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage(localize('giteaClone.configMissing', 'Gitea URL or Token is not configured.'));
        return;
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/user`, {
            headers: {
                'Authorization': `token ${token}`
            }
        });

        if (response.status === 200) {
            vscode.window.showInformationMessage(localize('giteaClone.authSuccess', `Authentication successful: ${response.data.username}`));
        }
    } catch (error) {
        vscode.window.showErrorMessage(localize('giteaClone.authFailed', 'Authentication failed.'));
        console.error(error);
    }
}

// Funktion zur Konfiguration der Gitea-Instanz und des Tokens über die Command Palette
async function configureGitea() {
    const instanceUrl = await vscode.window.showInputBox({
        prompt: localize('giteaClone.enterInstanceUrl', 'Enter Gitea instance URL'),
        placeHolder: 'https://your-gitea-instance.com'
    });

    const token = await vscode.window.showInputBox({
        prompt: localize('giteaClone.enterToken', 'Enter Gitea Personal Access Token'),
        placeHolder: 'token',
        password: true
    });

    if (instanceUrl && token) {
        await vscode.workspace.getConfiguration().update('gitea.instanceUrl', instanceUrl, vscode.ConfigurationTarget.Global);
        await vscode.workspace.getConfiguration().update('gitea.personalAccessToken', token, vscode.ConfigurationTarget.Global);
        vscode.window.showInformationMessage(localize('giteaClone.configUpdated', 'Gitea configuration updated.'));
    }
}

// Features

// Funktion zum Abrufen offener Pull Requests
async function getOpenPullRequests(): Promise<any[]> {
    const instanceUrl = vscode.workspace.getConfiguration().get<string>('gitea.instanceUrl');
    const token = vscode.workspace.getConfiguration().get<string>('gitea.personalAccessToken');

    if (!instanceUrl || !token) {
        vscode.window.showErrorMessage(localize('giteaClone.configMissing', 'Gitea URL oder Token ist nicht konfiguriert.'));
        return [];
    }

    // Aktuellen Git-Ordner abrufen
    const currentWorkspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri?.fsPath;
    if (!currentWorkspaceFolder) {
        vscode.window.showErrorMessage(localize('giteaClone.noValidRepo', 'Kein gültiges Git-Repository im aktuellen Arbeitsbereich gefunden.'));
        return [];
    }

    // Git-Repository-Informationen abrufen
    const { owner, repo } = await getRepoInfo(currentWorkspaceFolder);
    if (!owner || !repo) {
        vscode.window.showErrorMessage(localize('giteaClone.repoInfoError', 'Konnte Repository-Informationen nicht abrufen.'));
        return [];
    }

    try {
        const response = await axios.get(`${instanceUrl}/api/v1/repos/${owner}/${repo}/pulls`, {
            headers: {
                'Authorization': `token ${token}`
            },
            params: {
                state: 'open'
            }
        });

        if (response.status === 200) {
            return response.data;
        } else {
            vscode.window.showErrorMessage(localize('giteaClone.pullRequestsError', 'Fehler beim Abrufen der Pull Requests.'));
            return [];
        }
    } catch (error) {
        vscode.window.showErrorMessage(localize('giteaClone.apiError', 'Fehler bei der Verbindung zur Gitea API.'));
        console.error(error);
        return [];
    }
}

// Hilfsfunktion zum Abrufen von Repository-Informationen
async function getRepoInfo(folderPath: string): Promise<{ owner: string, repo: string }> {
    return new Promise((resolve, reject) => {
        exec(`git config --get remote.origin.url`, { cwd: folderPath }, (error, stdout) => {
            if (error || !stdout) {
                reject(localize('giteaClone.noRemoteUrl', 'Konnte Git-Remote-URL nicht abrufen.'));
            } else {
                let repoUrl = stdout.trim();

                // URL umwandeln, wenn sie im SSH-Format vorliegt
                if (repoUrl.startsWith('git@')) {
                    const parts = repoUrl.split(':');
                    const domain = parts[0].replace('git@', '');
                    const path = parts[1].replace('.git', '');
                    repoUrl = `https://${domain}/${path}`;
                } else {
                    repoUrl = repoUrl.replace('.git', '');
                }

                const [owner, repo] = repoUrl.split('/').slice(-2);
                resolve({ owner, repo });
            }
        });
    });
}

// Funktion zum Anzeigen offener Pull Requests
async function showOpenPullRequests() {
    const pullRequests = await getOpenPullRequests();

    if (pullRequests.length === 0) {
        vscode.window.showInformationMessage(localize('giteaClone.noOpenPRs', 'Keine offenen Pull Requests vorhanden.'));
        return;
    }

    const prItems = pullRequests.map(pr => ({
        label: `#${pr.number}: ${pr.title}`,
        description: `von ${pr.user.username}`,
        pr
    }));

    const selectedPr = await vscode.window.showQuickPick(prItems, {
        placeHolder: localize('giteaClone.selectPullRequest', 'Wähle einen Pull Request zum Anzeigen aus')
    });

    if (selectedPr) {
        // Öffne die URL des ausgewählten PRs im Browser
        vscode.env.openExternal(vscode.Uri.parse(selectedPr.pr.html_url));
    }
}


let extensionContext: vscode.ExtensionContext;

// Funktion zum Aktualisieren des PR-Statusleisten-Icons
let prStatusBarItem: vscode.StatusBarItem;

async function updatePRStatusBarItem(context: vscode.ExtensionContext) {
    try {
        const pullRequests = await getOpenPullRequests();

        if (!prStatusBarItem) {
            prStatusBarItem = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
            prStatusBarItem.command = 'gitea.showOpenPullRequests';
            prStatusBarItem.tooltip = localize('giteaClone.showOpenPullRequestsTooltip', 'Offene Pull Requests anzeigen');
            context.subscriptions.push(prStatusBarItem);
        }

        const prCount = pullRequests.length;
        prStatusBarItem.text = `$(git-pull-request) Gitea Open PRs: ${prCount}`;
        prStatusBarItem.show();
    } catch (error) {
        console.error('Fehler beim Aktualisieren des PR-Statusleisten-Icons:', error);
        if (prStatusBarItem) {
            prStatusBarItem.hide();
        }
    }
}

// Funktion zum Starten des PR-Statusleisten-Updaters
function startPRStatusBarItemUpdater(context: vscode.ExtensionContext) {
    updatePRStatusBarItem(context); // Initiales Update

    // Aktualisiere bei Fokuswechsel
    vscode.window.onDidChangeWindowState((windowState) => {
        if (windowState.focused) {
            updatePRStatusBarItem(context);
        }
    }, null, context.subscriptions);

    // Aktualisiere alle 5 Minuten
    setInterval(() => updatePRStatusBarItem(context), 300000); // 5 Minuten
}


// Aktivierungsfunktion des Plugins
export function activate(context: vscode.ExtensionContext) {
    // Registriert den Befehl zur Authentifizierung
    let authCommand = vscode.commands.registerCommand('gitea.authenticate', authenticateGitea);
    context.subscriptions.push(authCommand);

    // Registriert den Befehl zur Konfiguration
    let configCommand = vscode.commands.registerCommand('gitea.configure', configureGitea);
    context.subscriptions.push(configCommand);

    // Registriere den Befehl zum Erstellen eines Pull Requests
    let pullRequestCommand = vscode.commands.registerCommand('gitea.createPullRequest', createGiteaPullRequest);
    context.subscriptions.push(pullRequestCommand);

    // Statusbar-Icon sofort anzeigen
    const statusBar = addStatusBarIcon(); // Icon wird sofort beim Aktivieren angezeigt
    context.subscriptions.push(statusBar);

    // Befehl zum Klonen eines Repositories
    let cloneCommand = vscode.commands.registerCommand('gitea.cloneRepository', cloneGiteaRepository);
    context.subscriptions.push(cloneCommand);

    // Befehl zum Anzeigen offener Pull Requests
    let showPRCommand = vscode.commands.registerCommand('gitea.showOpenPullRequests', showOpenPullRequests);
    context.subscriptions.push(showPRCommand);

    // Statusleisten-Icon erstellen und Updater starten
    startPRStatusBarItemUpdater(context);
}

// Deaktivierungsfunktion des Plugins
export function deactivate() {}
